package filter
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Shader;
	import flash.display.ShaderJob;
	import flash.display.ShaderParameter;
	
	
	public class DisksFilter 
	{
		
		/** source image */
		protected var _input:BitmapData;
			
		/**	destination image */
		public var	_output:BitmapData;
		
		
		/**	embedded .pbj files	*/
		[Embed(source='shaders/Disks.pbj', mimeType = 'application/octet-stream')]
		protected var DisksShader:Class;
		
		/**	shader */
		protected var shader:Shader= new Shader(new DisksShader());
		
		/**	Constructor */
		public function DisksFilter(
			_input:BitmapData,
			_output:BitmapData)
		{
			input=_input;
			output=_output;
		}
		
		/**	Set the source bitmapdata */
		public function set input(value:BitmapData):void {
			this._input = value;
			update();
		}
		public function get input():BitmapData {
			return this._input;
		}
		
		/**	Set the destination bitmapdata */
		public function set output(value:BitmapData):void {
			this._output = value;
			update();
		}
		public function get output():BitmapData {
			return this._output;
		}
		
		
		/**	Applies the shader to image */
		public function update():void {
			if (_input==null || _output==null) return;
			
			shader.data.src.input = _input;
			var shaderJob:ShaderJob = new ShaderJob(shader, _output, _output.width, _output.height);
			
			_output.lock();
			//	apply the shader, true = waits here for completion
			shaderJob.start(true);
			_output.unlock();
		}
		
		
		/**	set pattern size */
		public function set size(value:Number):void {
			shader.data.size.value= [value]; 
			update();
		}
		
		[Bindable]
		public function get size():Number { 
			return shader.data.size.value[0];
		}
		
		
		/**	set radius */
		public function set radius(value:Number):void {
			shader.data.radius.value= [value];
			update();
		}
		
		[Bindable]
		public function get radius():Number { 
			return shader.data.radius.value[0];
		}
		
		
		/**	Sets the pattern rotation angle, value in degrees */
		public function set rotation(degrees:Number):void { 
			var radians:Number = degrees*Math.PI/180,
				cs:Number = Math.cos(radians),
				sn:Number = Math.sin(radians);
			//	set the rotation matrix
			shader.data.rotation.value  = [ cs,sn,-sn,cs ];
			//	and the reverse matrix
			// shader.data.rotationR.value = [ cs,-sn,sn,cs ];
			update();
		}
		
		
		/**	Base point for pattern origin, parameter point: any kind of object with properties x and y */
		public function set base(point:Object):void { 
			shader.data.base.value = [ point.x, point.y ]; 
			update();
		}
		public function set baseX(value:Number):void {
			shader.data.base.value[0] = value; 
			update();
		}
		public function set baseY(value:Number):void {
			shader.data.base.value[1] = value; 
			update();
		}
	}
}